<?php
# aquatube Gauge -- Version 10-May-2006
# Copyright (c) 2005, 2006 Jochen Kupperschmidt <webmaster@nwsnet.de>
# Released under the terms of the GNU General Public License
#  _                               _
# | |_ ___ _____ ___ _ _ _ ___ ___| |_
# |   | . |     | ._| | | | . |  _| . /
# |_|_|___|_|_|_|___|_____|___|_| |_|_\
#   http://homework.nwsnet.de/
#
# The aquatube Gauge can visualize percentages as a big or small aquatube
# image with an accordant water level.
# It makes use of PNG images with 8/4 bpp as well as transparency.
#
# Requires PHP 4.3.x or higher and GD library with PNG and Freetype support.
#
# The truetype font used is called 'ACME Explosive Bold' and freely available
# on various font-related websites.
# The aquatube images were kindly created and provided by Shoggy.


function loadPNG($filename)
{
    $img = imageCreateFromPNG($filename);
    if (! $img)
        trigger_error('Cannot initialize new gd image stream.', E_USER_ERROR);
    return $img;
}


# Configuration
define('FONT_FILE', 'acme_explosive_bold.ttf');


class Aquatube
{
    # Constructor. Load images from files.
    function Aquatube($insideFilename, $waterFilename, $frontFilename) {
        $this->inside = loadPNG($insideFilename);
        $this->water = loadPNG($waterFilename);
        $this->front = loadPNG($frontFilename);
        $this->level = 0;
    }

    # Set water level.
    function setLevel($level) {
        $level = abs((int) $level);
        $this->level = ($level > 100) ? 100 : $level;
    }

    # Fill in water.
    function fill($heightEmpty, $heightFull) {
        $range = abs($heightFull - $heightEmpty);
        $displacement = (int) ($range * (100 - $this->level) / 100);
        $x = (int) ((imageSX($this->inside) - imageSX($this->water)) / 2);
        imageCopyMerge(
            $this->inside,  # destination image
            $this->water,   # source image
            $x, $heightFull + $displacement,    # destination coordinates
            0, 0,   # source coordinates
            imageSX($this->water), imageSY($this->water),   # source dimensions
            75      # opacity
            );
    }

    # Put on front plate.
    function applyFront() {
        imageCopyMerge(
            $this->inside,  # destination image
            $this->front,   # source image
            0, 0,   # destination coordinates
            0, 0,   # source coordinates
            imageSX($this->front), imageSY($this->front),   # source dimensions
            100     # opacity
            );
    }

    # Write water level onto image.
    function applyLabel($size, $shadowDisplacement) {
        $text = $this->level . '%';

        # Compute text centering.
        $bbox = imageTTFBbox($size, 0, FONT_FILE, $text);
        $x = (int) ((imageSX($this->front) - abs(max($bbox[2], $bbox[4]))) / 2);
        $y = (int) ((imageSY($this->front) + abs(max($bbox[5], $bbox[7]))) / 2);

        # Draw text with shadow.
        $color = imageColorAllocate($this->inside, 245, 245, 245);
        $shadow = imageColorAllocate($this->inside, 21, 21, 21);
        imageTTFText($this->inside, $size, 0, $x + $shadowDisplacement, $y + $shadowDisplacement, $shadow, FONT_FILE, $text);
        imageTTFText($this->inside, $size, 0, $x, $y, $color, FONT_FILE, $text);
    }

    # Send image stream to stdout.
    function outputImage() {
        imagePNG($this->inside);
    }

    # Free memory.
    function cleanUp() {
        imageDestroy($this->inside);
        imageDestroy($this->water);
        imageDestroy($this->front);
    }
}


# Decide whether to create big or small aquatube image.
if ('small' == $_GET['size']) {
    $imgSize = 65;
    $heightEmpty = 48;
    $heightFull = 16;
    $fontSize = 7;
    $shadowDisplacement = 1;
} else {
    $imgSize = 250;
    $heightEmpty = 180;
    $heightFull = 60;
    $fontSize = 20;
    $shadowDisplacement = 2;
}

# Assemble aquatube image.
$at = new Aquatube(
    sprintf('aquatube_%1$dx%1$d_inside.png', $imgSize),
    sprintf('aquatube_%1$dx%1$d_water.png', $imgSize),
    sprintf('aquatube_%1$dx%1$d_front.png', $imgSize));
$at->setLevel($_GET['level']);
$at->fill($heightEmpty, $heightFull);
$at->applyFront();
$at->applyLabel($fontSize, $shadowDisplacement);

# Send image to browser.
header('Content-type: image/png');
$at->outputImage();
$at->cleanUp();
?>